import { Body, Controller, Get, Inject, Post, Query } from '@midwayjs/decorator';
import { ApiTags } from '@midwayjs/swagger';
import { UsualService } from '../service/usual.service';
import { ExportByLab, FilePath, InspectCard, personTotal } from '../interface/usual.interface';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { getModelWithString } from '@typegoose/typegoose';
import get = require('lodash/get');
import upperFirst = require('lodash/upperFirst');
import head = require('lodash/head');
import isObject = require('lodash/isObject');
import groupBy = require('lodash/groupBy');
const assert = require('assert');
@ApiTags(['工具'])
@Controller('/usual')
export class UsualController {
  @Inject()
  service: UsualService;

  @Post('/exportByLab')
  async exportByLab(@Body() body: ExportByLab) {
    const { config, targets, query } = body;
    assert(config, '缺少导出配置,无法进行导出!');
    assert(isObject(config), '配置信息数据格式错误,无法进行导出!');
    // 1.建立excel文件
    const fileName = `信息导出-${new Date().getTime()}`;
    const { fullPath, downPath }: FilePath = this.service.makeExcelFullRealPath(fileName);
    const workbook = this.service.makeExcelFile(fullPath);
    // 2.找到范围内的实验室
    const labQuery: any = {};
    if (get(query, 'region')) labQuery.region = get(query, 'region');
    const labList = await this.service.getLabInfo(targets, labQuery);
    const lab_ids = labList.map(i => i._id);
    // 3.解析配置
    const tables = Object.keys(config);
    for (const table of tables) {
      const model = getModelWithString(upperFirst(table));
      const modelConfig = await this.service.readModel(table);
      // 获取选择的设置,排序;因为后面分组,会打乱顺序
      let setting = config[table];
      setting = setting.map((i, index) => ({ ...i, index }));
      // 根据当前选择内容,获取sheet名
      const sheetName = get(head(setting), 'table_zh');
      if (!sheetName) continue;
      // 创建对应项目的sheet
      const sheet = workbook.addWorksheet(sheetName);
      // 设置表头
      const firstLine = setting.map(i => i.key_zh);
      sheet.addRow(firstLine).commit();
      // 查询所有数据
      const sourceData = await this.service.getExportData({ table, lab_ids, query, targetModel: model, modelConfig });
      // 将setting区分开,有dict类的,有ref表关联类的
      // key:def=>默认使用类;dict=>字典表类;xxxx=>各种表关联类
      const group = groupBy(setting, v => {
        const { dict, ref } = v;
        if (ref) return ref;
        else if (dict) return 'dict';
        return 'def';
      });
      for (const d of sourceData) {
        const arr = await this.service.setData(group, d);
        sheet.addRow(arr).commit();
      }
    }
    await workbook.commit();
    return downPath;
  }

  @Get('/inspectCard')
  async inspectCard(@Query() inspectCard: InspectCard) {
    await this.service.inspectCard(inspectCard);
    return 'ok';
  }

  @Get('/person')
  async query(@Query() filter: personTotal, @Query('skip') skip: number, @Query('limit') limit: number) {
    const list = await this.service.personTotal(filter, { skip, limit });
    const data = [];
    for (const i of list.arr) {
      data.push({ _id: i._id, name: i.name });
    }
    const total = list.total;
    return { data, total };
  }

  @Get('/readModel')
  async readModel(@Query('table') table: string) {
    const result = await this.service.readModel(table);
    return result;
  }

  @Post('/readModels')
  async readModels(@Body('tables') tables: Array<any>) {
    const result = [];
    for (const t of tables) {
      const { value } = t;
      const r = await this.service.readModel(value);
      result.push({ value: t.value, label: t.label, config: r });
    }
    return result;
  }

  @Post('/exports')
  async exports() {
    throw new ServiceError('请使用 以实验室为中心的导出功能', FrameworkErrorEnum.SERVICE_FAULT);
  }

  @Post('/importBasic')
  async importBasic() {
    throw new ServiceError('暂不开放此功能', FrameworkErrorEnum.SERVICE_FAULT);
  }
}
