import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { getModelWithString, ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { Basic } from '../entity/lab/basic.entity';
import { QDTO_basic } from '../interface/basic.interface';
import get = require('lodash/get');
import pick = require('lodash/pick');
import upperFirst = require('lodash/upperFirst');
import round = require('lodash/round');
import divide = require('lodash/divide');
import { Build } from '../entity/lab/build.entity';
import { User } from '../entity/lab/user.entity';
type modelType = ReturnModelType<typeof Basic>;
@Provide()
export class BasicService extends BaseService<modelType> {
  @InjectEntityModel(Basic)
  model: modelType;
  @InjectEntityModel(Build)
  buildModel: ReturnModelType<typeof Build>;

  @InjectEntityModel(User)
  userModel: ReturnModelType<typeof User>;

  /** 与当前用户关联 */
  async afterCreate(data) {
    const user = this.ctx.user;
    if (!user) {
      await this.delete(data._id);
      throw new ServiceError('未找到当前用户信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    }
    const user_id = get(user, '_id');
    if (!user_id) throw new ServiceError('未找到当前用户的信息,请重新登陆', FrameworkErrorEnum.NOT_FOUND_DATA);
    const ud = await this.userModel.findOne({ user_id });
    if (ud) {
      await this.userModel.updateOne({ _id: ud._id }, { lab_id: data._id });
    } else {
      await this.delete(data._id);
      throw new ServiceError('未找到当前用户信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    }
  }

  async percent(condition: QDTO_basic, { skip = 0, limit = 0 }) {
    const data = await this.model.find(condition.getFilter()).sort({ 'meta.createdAt': -1 }).skip(skip).limit(limit).lean();
    const total = await this.model.count(condition.getFilter());
    let list = await this.computedBasic(data);
    list = await this.computedBuild(list);
    list = await this.computedOthers(list);
    list = this.computedResult(list);
    return { data: list, total };
  }

  /**
   *根据之前的结果就
   * @param {Array} data 计算获得接口
   */
  computedResult(data) {
    data = data.map(i => {
      const { have, cTotal, ...others } = i;
      const obj = { ...others };
      obj.progress = round(divide(have, cTotal) * 100, 2);
      return obj;
    });
    return data;
  }

  /**
   * 累加,实验室 其他的数据
   * @param {Array} data 实验室 计算基础信息和平台建设后 的数据
   */
  async computedOthers(data) {
    const list = [
      { label: '研究方向', value: 'direction' },
      { label: '参加单位', value: 'company' },
      { label: '固定人员', value: 'fixedpersonnel' },
      { label: '流动人员', value: 'flowpersonnel' },
      { label: '学术委员会', value: 'committee' },
      { label: '科研项目', value: 'science' },
      { label: '项目经费', value: 'fixInput' },
      { label: '代表性论文', value: 'paper' },
      { label: '其他代表性成果', value: 'otherAward' },
      { label: '获奖情况', value: 'award' },
      { label: '成果转化', value: 'achieve' },
      { label: '开放服务', value: 'technicalservice' },
      { label: '设备及场地', value: 'equipment' },
      // { label: '知识产权,标准', value: 'tecAward' },
      { label: '省部级及以上活动', value: 'sjactivity' },
      { label: '科普活动', value: 'scienceactivities' },
      { label: '发表,提交报告', value: 'report' },
      { label: '人才称号', value: 'personnelname' },
      { label: '职称晋升', value: 'title' },
      { label: '博硕培养', value: 'doctor' },
      { label: '重要成果', value: 'outcome' },
    ];
    for (const d of data) {
      const { _id } = d;
      for (const c of list) {
        const { value: table } = c;
        // // 设置model
        const targetModel = getModelWithString(upperFirst(table));
        if (!targetModel) {
          console.log(table);
          continue;
        }
        const r = await targetModel.count({ lab_id: _id });
        if (r > 0) d.have = d.have + 1;
        d.cTotal = d.cTotal + 1;
      }
    }
    return data;
  }

  /**
   * 累加 实验室 平台建设后的数据
   * @param {Array} data 实验室 计算基础信息后 的数据
   */
  async computedBuild(data) {
    const modelConfig: any = get(this.buildModel, 'schema.tree');
    const midObj = pick(modelConfig, ['summary', 'purport', 'base', 'build', 'plan']);
    const keys = Object.keys(midObj);
    for (const d of data) {
      const r = await this.buildModel.findOne({ lab_id: d._id });
      d.cTotal = d.cTotal + keys.length;
      const l = keys.filter(f => get(r, f));
      d.have = d.have + l.length;
    }
    return data;
  }

  /**
   * 根据实验室信息,查询每个实验室都填写了多少项
   * @param {Array} data 实验室信息
   */
  async computedBasic(data) {
    const modelConfig: any = get(this.model, 'schema.tree');
    const arr = [];
    delete modelConfig.status;
    delete modelConfig.is_show;
    delete modelConfig.ss_laboratory;
    delete modelConfig.is_alike;
    for (const key in modelConfig) {
      const d = modelConfig[key];
      if (!d.props) {
        arr.push({ column: key, name: d.zh });
        continue;
      }
      for (const p of d.props) {
        const { key: pk, zh } = p;
        arr.push({ column: `${key}.${pk}`, name: `${d.zh}.${zh}` });
      }
    }
    const rData = [];
    const cTotal = arr.length;
    for (const d of data) {
      const { name, lab_person, lab_phone, _id } = d;
      let have = 0;
      for (const c of arr) {
        const { column } = c;
        const res = get(d, column);
        if (res) have++;
      }
      rData.push({ _id, name, have, cTotal, lab_person, lab_phone });
    }
    return rData;
  }
}
