import { Provide, Inject } from '@midwayjs/decorator';
import { Context } from '@midwayjs/koa';
import get = require('lodash/get');
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { Paper } from '../entity/lab/achievements/paper.entity';
import { OtherAward } from '../entity/lab/achievements/otherAward.entity';
import { Doctor } from '../entity/lab/team/doctor.entity';
import { Technicalservice } from '../entity/lab/achievements/technicalservice.entity';
import { Achieve } from '../entity/lab/achievements/achieve.entity';
import { Science } from '../entity/lab/achievements/science.entity';
import { Award } from '../entity/lab/achievements/award.entity';
const assert = require('assert');
@Provide()
export class StatisticsService {
  @Inject()
  ctx: Context;

  @InjectEntityModel(Paper)
  papermodel: ReturnModelType<typeof Paper>;

  @InjectEntityModel(OtherAward)
  otherAwardmodel: ReturnModelType<typeof OtherAward>;

  @InjectEntityModel(Achieve)
  achievemodel: ReturnModelType<typeof Achieve>;

  @InjectEntityModel(Doctor)
  doctormodel: ReturnModelType<typeof Doctor>;

  @InjectEntityModel(Technicalservice)
  technicalservicemodel: ReturnModelType<typeof Technicalservice>;

  @InjectEntityModel(Science)
  sciencemodel: ReturnModelType<typeof Science>;

  @InjectEntityModel(Award)
  awardmodel: ReturnModelType<typeof Award>;
  /**
   * 成果统计
   * @param {String} lab_id 实验室id
   */
  async achieve(lab_id) {
    assert(lab_id, '缺少实验室信息');
    const obj: any = {};
    // p1 发表论文统计
    obj.p1 = await this.papermodel.count(lab_id);
    // p2,3,6-10 其他成果分类统计
    const otherAwards = await this.otherAwardmodel.aggregate([{ $match: lab_id }, { $project: { type: 1 } }, { $group: { _id: '$type', total: { $sum: 1 } } }]);
    const otherAwardMeta = this.getOtherAwardMeta();
    for (const meta of otherAwardMeta) {
      const { label, key } = meta;
      const findRes = otherAwards.find(f => f._id.includes(label));
      if (!findRes) obj[key] = 0;
      else {
        const { total = 0 } = findRes;
        obj[key] = total;
      }
    }
    // p4 科研项目
    obj.p4 = await this.sciencemodel.count(lab_id);
    // p5 获奖情况
    obj.p5 = await this.awardmodel.count(lab_id);
    // p11-13 人才培养
    const doctors = await this.doctormodel.aggregate([{ $match: lab_id }, { $project: { education: 1 } }, { $group: { _id: '$education', total: { $sum: 1 } } }]);
    const doctorsMeta = this.getDoctorsMeta();
    for (const meta of doctorsMeta) {
      const { value, key } = meta;
      if (value) {
        const fincRes = doctors.find(f => f._id === value);
        if (!fincRes) {
          obj[key] = 0;
          continue;
        }
        const { total } = fincRes;
        obj[key] = total;
      } else {
        // 这里用动态排除方式做,以免之后改了多处地方
        const haveValueMetas = doctorsMeta.filter(f => f.value);
        const total = doctors.reduce((p, n) => {
          const { _id, total } = n;
          const isHaveValue = haveValueMetas.find(f => f.value === _id);
          let number = 0;
          if (!isHaveValue) number = total;
          return p + number;
        }, 0);
        obj[key] = total;
      }
    }
    // p14,15 科技交流活动,科技人才交流
    const techData = await this.technicalservicemodel.find(lab_id, { number: 1 });
    obj.p14 = techData.length;
    obj.p15 = techData.reduce((p, n) => p + get(n, 'number', 0), 0);
    // p16,17 科技成果转化数/技术合作与协作
    const achieves = await this.achievemodel.aggregate([{ $match: lab_id }, { $project: { way: 1 } }, { $group: { _id: '$way', total: { $sum: 1 } } }]);
    obj.p16 = achieves.reduce((p, n) => p + (n.total || 0), 0);
    const mid = achieves.filter(f => f._id !== '4');
    obj.p17 = mid.reduce((p, n) => p + (n.total || 0), 0);
    return obj;
  }

  /**
   * 用的是value判断
   */
  getDoctorsMeta() {
    return [
      { value: '1', key: 'p11' }, // 硕士
      { value: '2', key: 'p12' }, // 博士
      { key: 'p13' }, // 没有label说明除了上面2个外的结果
    ];
  }

  /**
   * 使用label查找,没用value;这玩应随便改,记得改的时候把上面对应的判断也改了
   */
  getOtherAwardMeta() {
    return [
      { label: '专著', key: 'p2' },
      { label: '标准', key: 'p3' },
      { label: '发明专利', key: 'p6' },
      { label: '实用新型专利', key: 'p7' },
      { label: '软件著作权', key: 'p8' },
      { label: '新药证书', key: 'p9' },
      { label: '医疗器械注册证', key: 'p10' },
    ];
  }
}
