import { Config, Inject, Provide } from '@midwayjs/decorator';
import { getModelWithString, ReturnModelType } from '@typegoose/typegoose';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { FilePath, FilesConfig, GetExportData, InspectCard, personTotal } from '../interface/usual.interface';
import * as fs from 'node:fs';
import * as Path from 'node:path';
import { RegionService } from './region.service';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { Basic } from '../entity/lab/basic.entity';
import { Types } from 'mongoose';
import get = require('lodash/get');
import isObject = require('lodash/isObject');
import pick = require('lodash/pick');
import has = require('lodash/has');
import upperFirst = require('lodash/upperFirst');
import isArray = require('lodash/isArray');
import head = require('lodash/head');
import orderBy = require('lodash/orderBy');
import { Fixedpersonnel } from '../entity/lab/person/fixedpersonnel.entity';
import { Flowpersonnel } from '../entity/lab/person/flowpersonnel.entity';
import { DictData } from '../entity/basic/dictData.entity';
import { concat } from 'lodash';
const sep = Path.sep;
const Excel = require('exceljs');
const ObjectId = Types.ObjectId;
@Provide()
export class UsualService {
  @Config('filesConfig')
  filesConfig: FilesConfig;

  @Inject()
  regionService: RegionService;

  @InjectEntityModel(Basic)
  basicModel: ReturnModelType<typeof Basic>;

  @InjectEntityModel(Fixedpersonnel)
  fpModel: ReturnModelType<typeof Fixedpersonnel>;

  @InjectEntityModel(Flowpersonnel)
  flpModel: ReturnModelType<typeof Flowpersonnel>;

  @InjectEntityModel(DictData)
  dictDataModel: ReturnModelType<typeof DictData>;

  /**
   * 读取model的设置
   * @param table 要读取的model
   */
  async readModel(table: string): Promise<Array<any>> {
    const model = getModelWithString(upperFirst(table));
    let data: any = get(model, 'schema.tree');
    if (!(data && isObject(data))) {
      throw new ServiceError('未找到数据源', FrameworkErrorEnum.BAD_ARGS);
    }
    data = JSON.parse(JSON.stringify(data));
    const keys = Object.keys(data);
    let arr = [];
    for (const key of keys) {
      const obj = { key, ...pick(data[key], ['zh', 'ref', 'props', 'dict', 'exportQuery', 'exportFormat']) };
      // props 是针对object,将其拿出来平铺上,进行key和zh的组合,将原来的干掉
      if (obj.props) {
        const { zh, key, props } = obj;
        for (const p of props) {
          const { zh: pzh, key: pkey } = p;
          arr.push({ zh: `${zh}-${pzh}`, key: `${key}.${pkey}` });
        }
      } else arr.push(obj);
    }
    arr = arr.filter(f => has(f, 'zh'));
    const has_lab = arr.findIndex(f => f.key === 'lab_id');
    if (has_lab > -1) {
      const lab_idColumn = arr.find(f => f.key === 'lab_id');
      arr.splice(has_lab, 1);
      arr.unshift(lab_idColumn);
    }
    // 将ref的内容展开
    for (const config of arr) {
      let ref: string;
      if (config.key === 'lab_id') ref = 'Basic';
      else ref = config.ref;
      if (!ref) continue;
      if (ref === 'person') continue;
      const refConfig = await this.readModel(ref);
      config.refConfig = refConfig;
    }
    return arr;
  }
  /**
   * 查询固定人员与流动人员
   * @param query 查询条件
   */
  async personTotal(query: personTotal, { skip = 0, limit = 0 }) {
    const fp: any = await this.fpModel
      .find(query.getFilter())
      .skip(skip)
      .limit(limit / 2)
      .lean();
    const flp: any = await this.flpModel
      .find(query.getFilter())
      .skip(skip)
      .limit(limit / 2)
      .lean();
    const arr = concat(fp, flp);
    const total1 = await this.fpModel.count(query.getFilter());
    const total2 = await this.flpModel.count(query.getFilter());
    const total = total1 + total2;
    return { arr, total };
  }
  /**
   * 查询固定人员与流动人员身份证号是否重复
   * @param query 查询条件
   */
  async inspectCard(query: InspectCard) {
    const { table, card, lab_id } = query;
    const model = getModelWithString(upperFirst(table));
    const data = await model.findOne({ card });
    if (data) {
      if (data.lab_id === lab_id) throw new ServiceError('该身份证号已存于当前实验室中', FrameworkErrorEnum.BAD_BODY);
      else throw new ServiceError('该身份证号存在于其他实验室中', FrameworkErrorEnum.BAD_BODY);
    }
  }
  /**
   * 组织导出数据
   * @param group 分组后的表设置数据
   * @param data 导出的数据
   */
  async setData(group, data) {
    // 根据group,开始整理数据
    let arr = [];
    for (const keyType in group) {
      const s = group[keyType];
      if (keyType === 'def') {
        // 数组中都是常规处理的,直接循环数组取出来即可
        for (const sc of s) {
          let obj = {};
          const { key, index, exportFormat } = sc;
          obj = { value: exportFormat ? exportFormat(data[key]) : get(data, key), index };
          arr.push(obj);
        }
      } else if (keyType === 'dict') {
        // 数组都是字典表来的.得去各个字典表取数据然后存值
        for (const sc of s) {
          let obj = {};
          const { key, index, dict } = sc;
          const value = await this.getDataFromDict(dict, data[key]);
          obj = { value: value || '', index };
          arr.push(obj);
        }
      } else if (keyType === 'person') {
        for (const sc of s) {
          const { key, index } = sc;
          const refData = await this.getDataFromRef(keyType, data[key]);
          let obj = {};
          obj = { value: refData, index };
          arr.push(obj);
        }
      } else {
        // 从各个表来的,keyType就是表名,每个数组都是一个表的字段
        const refKey = get(head(s), 'ref_key') as string;
        const props = s.map(i => i.key);
        const refData = await this.getDataFromRef(keyType, data[refKey], props);
        for (const sc of s) {
          let obj = {};
          const { key, index } = sc;
          obj = { value: refData[key], index };
          arr.push(obj);
        }
      }
    }
    arr = orderBy(arr, ['index'], ['asc']);
    return arr.map(i => i.value);
  }

  /**
   * 获取导出数据(参数变为结构模式)
   */
  async getExportData({ table, lab_ids, query, targetModel, modelConfig }: GetExportData) {
    let sourceData = [];
    if (table === 'basic') sourceData = await this.basicModel.find({ _id: lab_ids });
    else {
      const queryList = modelConfig.filter(f => f.exportQuery);
      const toQuery: any = {};
      for (const q of queryList) {
        const { key, exportQuery } = q;
        if (query[exportQuery]) toQuery[key] = query[exportQuery];
      }
      sourceData = await targetModel.find({ lab_id: lab_ids, ...toQuery });
    }
    return sourceData;
  }

  /**
   * 根据实验室id列表返回实验室(数组为空则返回全部)
   * @param {Array} list 实验室id列表
   * @param {Object} query 实验室id查询条件:目前只有region一个,导出指定地区的数据
   */
  async getLabInfo(list, query: any = {}) {
    const querys: any = {};
    if (isArray(list) && list.length > 0) querys._id = list;
    // 如果指定了地区,那就查询指定地区的实验室,结合目标实验室一起查询
    if (query.region) {
      const regionRange = await this.regionService.query(query.region);
      querys.region = regionRange.map(i => i.code);
    }
    const res = await this.basicModel.find(querys, { name: 1 });
    return res;
  }

  /**
   * 根据 字典类型 和 值, 获取选择的字典中文
   * @param {String} type 字典类型
   * @param {String} value 值
   */
  async getDataFromDict(type, value) {
    const dictData = await this.dictDataModel.findOne({ dict_type: type, dict_value: value }, { dict_label: 1 });
    if (dictData) return dictData.dict_label;
  }

  /**
   * 获取关联内容
   * @param {String} ref 表名
   * @param {String} value 查询用的值
   * @param {Array} props 选取查询结果的内容
   * @param {String} column 查询用的字段,默认_id
   */
  async getDataFromRef(ref, value, props?, column = '_id') {
    if (ref === 'person') {
      const ids = isArray(value) ? value.map(i => new ObjectId(i)) : new ObjectId(value);
      const fixedList = await this.fpModel.find({ _id: ids }, { name: 1 });
      const flowList = await this.flpModel.find({ _id: ids }, { name: 1 });
      let newArr: any = [...fixedList, ...flowList];
      newArr = newArr.map(i => i.name);
      return newArr.join(';');
    }
    const mod = getModelWithString(upperFirst(ref));
    if (!mod) return;
    const query: any = {};
    if (column === '_id') query._id = new ObjectId(value);
    else query[column] = value;
    const projection = {};
    for (const p of props) {
      projection[p] = 1;
    }
    const data = await mod.findOne(query, projection);
    return pick(data, props);
  }

  /**
   * 生成excel,返回工作本(workbook)
   * @param {String} fullPath 文件全路径
   */
  makeExcelFile(fullPath) {
    if (!fullPath) throw new ServiceError('未获取到文件全路径,无法进行导出', FrameworkErrorEnum.SERVICE_FAULT);
    try {
      const options = { filename: fullPath };
      const workbook = new Excel.stream.xlsx.WorkbookWriter(options);
      return workbook;
    } catch (error) {
      throw new ServiceError('生成excel失败,无法进行导出', FrameworkErrorEnum.SERVICE_FAULT);
    }
  }

  /**
   * 生成文件全路径
   * @param {String} fileName 文件名称
   */
  makeExcelFullRealPath(fileName): FilePath {
    if (!fileName) throw new ServiceError('未获取到文件名,无法进行导出', FrameworkErrorEnum.SERVICE_FAULT);
    const { rPath, path } = this.checkPath();
    fileName = `${fileName}.xlsx`;
    const fullPath = `${rPath}${fileName}`; // 实际全路径
    const downPath = `${path}${fileName}`; // 下载路径
    return { fullPath, downPath };
  }

  /**
   * 检查路径,创建路径;返回前端接收路径
   * @param {Array} filesPath 地址拆开 例如: ['www','dir','example']
   * @return {Object} path 短地址;rPath 文件真实路径
   */
  checkPath(filesPath = []) {
    const { root_path, project, export_dir } = this.filesConfig;
    const arr = [root_path, project, export_dir, ...filesPath];
    let rPath = '';
    for (const i of arr) {
      rPath = `${rPath}${i}${sep}`;
      if (!fs.existsSync(rPath)) {
        fs.mkdirSync(rPath);
      }
    }
    arr.shift();
    arr.unshift('files');
    const path = `/${arr.join('/')}/`;
    return { path, rPath };
  }
}
